package com.zsc.edu.dify.framework.security.flux;
import com.zsc.edu.dify.common.util.TreeUtil;
import com.zsc.edu.dify.exception.StateException;
import com.zsc.edu.dify.framework.security.UserDetailsImpl;
import com.zsc.edu.dify.modules.system.entity.Dept;
import com.zsc.edu.dify.modules.system.entity.Menu;
import com.zsc.edu.dify.modules.system.entity.Role;
import com.zsc.edu.dify.modules.system.entity.UserRole;
import com.zsc.edu.dify.modules.system.repo.flux.*;
import com.zsc.edu.dify.modules.system.service.DeptService;
import lombok.AllArgsConstructor;
import org.springframework.security.core.userdetails.ReactiveUserDetailsService;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Mono;

import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @author zhuang
 */
@AllArgsConstructor
@Service
public class FluxUserDetailServiceImpl implements ReactiveUserDetailsService {

    private final UserFluxRepository userRepo;
    private final MenuFluxRepository menuRepository;
    private final RoleFluxRepository roleFluxRepository;
    private final UserRoleFluxRepository userRoleFluxRepository;
    private final DeptService deptService;
    @Override
    public Mono<UserDetails> findByUsername(String username) {
        return userRepo.findByUsername(username)
                .flatMap(user -> {
                    if (!user.getEnableState()) {
                        return Mono.error(new StateException("用户 '" + username + "' 已被禁用！请联系管理员"));
                    }
                    // 先获取UserRole列表，并从中提取roleIds
                    return userRoleFluxRepository.findByUserId(user.getId()).collectList()
                            .flatMap(userRoles -> Mono.zip(
                                roleFluxRepository.findByDeptId(user.getDeptId()),
                                deptService.listFluxTree(user.getDeptId()),
                                menuRepository.findByRoleId(user.getRoleId())
                            ))
                            .flatMap(tuple -> {
                                Role role = (Role) tuple.getT1();
                                List<Dept> depts = (List<Dept>) tuple.getT2();
                                List<Menu> menus = tuple.getT3();

                                user.setRole(role);

                                Set<Long> dataScopeDeptIds = depts.stream()
                                        .map(Dept::getId)
                                        .collect(Collectors.toSet());
                                user.setDataScopeDeptIds(dataScopeDeptIds);

                                Set<String> permissions = menus.stream()
                                        .map(Menu::getPermissions)
                                        .collect(Collectors.toSet());

                                return Mono.just(UserDetailsImpl.from(user, permissions));
                            });
                });
    }



//    @Override
//    public Mono<UserDetails> findByUsername(String username) {
//        return userRepo.findByUsername(username)
//                .flatMap(user -> menuRepository.findByRoleId(user.getRoleId())
//                        .map(ids-> userRoleFluxRepository.findByUserId(user.getId()))
//                        .map(roles-> roleFluxRepository.findByIdIn(ids))
//                        .map(menus -> {
//                            Set<String> permissions = menus.stream().map(Menu::getPermissions).collect(Collectors.toSet());
//                            return UserDetailsImpl.from(user, permissions);
//                        })
//                );
//    }

}
