package com.zsc.edu.dify.modules.dify.controller;

import com.zsc.edu.dify.modules.dify.repo.AppEntityRepository;
import io.github.guoshiqiufeng.dify.chat.DifyChat;
import io.github.guoshiqiufeng.dify.chat.dto.request.ChatMessageSendRequest;
import io.github.guoshiqiufeng.dify.chat.dto.request.MessageConversationsRequest;
import io.github.guoshiqiufeng.dify.chat.dto.response.ChatMessageSendCompletionResponse;
import io.github.guoshiqiufeng.dify.chat.dto.response.MessageConversationsResponse;
import io.github.guoshiqiufeng.dify.core.pojo.DifyPageResult;
import jakarta.annotation.Resource;
import org.springframework.http.MediaType;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Flux;

/**
 * @author yanghq
 * @version 1.0
 * @since 2025/3/25 10:48
 */
@RestController
@RequestMapping("/v1/chat")
public class V1ChatController {

    @Resource
    private DifyChat difyChat;
    @Resource
    private AppEntityRepository appEntityRepository;

    /**
     * 发送消息（流式）
     *
     * @param sendRequest 消息参数 （可以自定义参数，调用 difyChat 实例时重新组装即可），
     *                    用户 id可以改为从上下文（token）获取，
     *                    apikey 建议在数据库进行存储，前端调用时传智能体 id，从数据库查询
     */
    @PostMapping(value = "/completions/{appId}", produces = MediaType.TEXT_EVENT_STREAM_VALUE)
    @PreAuthorize("hasAuthority('difyChat:query')")
    public Flux<ChatMessageSendCompletionResponse> sendChatMessageStream(@RequestBody ChatMessageSendRequest sendRequest, @PathVariable String appId) {
        sendRequest.setApiKey(appEntityRepository.selectApiKey(appId));
        return difyChat.sendChatMessageStream(sendRequest);
    }

    /**
     * 获取会话列表
     *
     * @param request 请求参数
     * @return 会话列表
     */
    @PostMapping("/conversations")
    @PreAuthorize("hasAuthority('difyChat:query')")
    public DifyPageResult<MessageConversationsResponse> conversations(@RequestBody MessageConversationsRequest request) {
        request.setApiKey("app-mM2UGTE5QVPLCwGvwifnV0g7");
        return difyChat.conversations(request);
    }

    /**
     * 停止流式消息
     *
     * @param taskId  任务id
     * @param userId  用户id
     */
    @PatchMapping("/stopMessagesStream")
    @PreAuthorize("hasAuthority('difyChat:update')")
    public void stopMessagesStream( String taskId, String userId) {
        difyChat.stopMessagesStream("app-mM2UGTE5QVPLCwGvwifnV0g7", taskId, userId);
    }

    /**
     * 删除会话
     *
     * @param conversationId 会话id
     * @param userId         用户id
     */
    @DeleteMapping("/messages/suggested")
    @PreAuthorize("hasAuthority('difyChat:delete')")
    public void deleteConversation(String conversationId, String userId) {
        difyChat.deleteConversation(conversationId, "app-mM2UGTE5QVPLCwGvwifnV0g7", userId);
    }
}