package com.zsc.edu.dify.modules.dify.controller;

import io.github.guoshiqiufeng.dify.core.pojo.DifyPageResult;
import io.github.guoshiqiufeng.dify.workflow.DifyWorkflow;
import io.github.guoshiqiufeng.dify.workflow.dto.request.WorkflowLogsRequest;
import io.github.guoshiqiufeng.dify.workflow.dto.request.WorkflowRunRequest;
import io.github.guoshiqiufeng.dify.workflow.dto.response.*;
import jakarta.annotation.Resource;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Flux;

@RestController
@RequestMapping("/v1/work")
public class V1WorkflowController {

    @Resource
    private DifyWorkflow difyWorkflow;

    /**
     * 运行工作流
     *
     * @param request
     * @return
     */
    @PostMapping("/run")
    @PreAuthorize("hasAuthority('difyWorkFlow:query')")
    public WorkflowRunResponse runWorkflow(@RequestBody WorkflowRunRequest request) {
        request.setApiKey("app-ZpkQM6yy767oUTfNSBYq65nB");
        return difyWorkflow.runWorkflow(request);
    }

    /**
     * 运行工作流(流式)
     *
     * @param request
     * @return
     */
    @PostMapping("/run/stream")
    public Flux<WorkflowRunStreamResponse> runWorkflowStream(@RequestBody WorkflowRunRequest request) {
        request.setApiKey("app-ZpkQM6yy767oUTfNSBYq65nB");
        return difyWorkflow.runWorkflowStream(request);
    }

    /**
     * 停止工作流(流式)
     *
     * @param taskId
     * @param userId
     * @return
     */
    @PatchMapping("/stop")
    public WorkflowStopResponse stopWorkflowStream( String taskId, String userId){
        return difyWorkflow.stopWorkflowStream("app-mM2UGTE5QVPLCwGvwifnV0g7", taskId, userId);
    }

    /**
     * 获取工作流信息
     *
     * @param workflowRunId
     * @return
     */
    @GetMapping("/info")
    @PreAuthorize("hasAuthority('difyWorkFlow:query')")
    public WorkflowInfoResponse info(String workflowRunId) {
        return difyWorkflow.info(workflowRunId, "app-ZpkQM6yy767oUTfNSBYq65nB");
    }

    /**
     * 获取工作流日志
     *
     * @param request
     * @return
     */
    @PostMapping("/logs")
    @PreAuthorize("hasAuthority('difyWorkFlow:query')")
    public DifyPageResult<WorkflowLogs> logs(@RequestBody WorkflowLogsRequest request) {
        request.setApiKey("app-ZpkQM6yy767oUTfNSBYq65nB");
        return difyWorkflow.logs(request);
    }
}
